#!/usr/bin/env python
# coding: utf-8

# ESA (C) 2000-2022
#
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.


import numpy as np
import matplotlib.pyplot as plt
import os
import sys
import re
import glob
from matplotlib.ticker import PercentFormatter
from matplotlib.backends.backend_pdf import PdfPages
import warnings
import datetime


def get_means_of_lists(full_list):
    """
    Calculates the average values of several given lists element-wise.

    Args:
        full_list: the list of lists to find averages.

    Output:
        avg_list: a list containing the average of elements.
    """
    
    arrays = [np.array(x) for x in full_list]
    avg_list = [np.mean(k) for k in zip(*arrays)]
    avg_list = np.array(avg_list)
    
    return avg_list

    
def error_analysis(error_list):
    """
    Computes the standard deviation from the list of error lists/arrays given.

    Args:
        error_list: the list containing all the error lists.

    Output:
       err_list: the final list of standard deviation. 
    """

    arrays = [np.array(x) for x in error_list]
    err_list = [np.std(k) for k in zip(*arrays)]
    err_list = np.array(err_list)

    return err_list


def calc_avg(avg_file):
    """
    Loads the file corresponding to the average/final data. 

    Args:
        avg_file: the path to the text file containing the data.

    Output:
        vals: a numpy array with the numerical data.
    """
    
    try:
        vals = np.loadtxt(avg_file)
    except FileNotFoundError:
        warnings.warn('Could not open {}.'.format(avg_file))

    return vals


def main(filename):
    """
    Makes a series of plots related to the ompsfprofile SAS task. This script is launched directly from ompsfprofile_mod.f90.

    Args:
        filename: the name to use to save the file.
    
    Output:
        None
    """

    # Glob: gather all files:
    # Accumulative sum:
    x_sum_files = glob.glob('xdatasum*')
    x_sum_files.sort()
    y_sum_files = glob.glob('ydatasum*')
    y_sum_files.sort()

    # Get total number of figures assuming all of them have the accumulated sum:
    n_figures = len(x_sum_files)
    
    if n_figures == 0:
        warnings.warn('No accumulative sum temporary files found. Quitting...')
        return None

    # Calibration curve 
    x_cal_files = glob.glob('xdatacal*')
    y_cal_files = glob.glob('ydatacal*')
    x_cal_files.sort()
    y_cal_files.sort()

    # Final files:
    final_files = glob.glob('*datafinal_*')

    # Mean files:
    mean_files = glob.glob('*datamean_*')
    # Average curve
    x_avg_files = glob.glob('xdatadots*')
    y_avg_files = glob.glob('ydatadots*')
    e_avg_files = glob.glob('edatadots*')
    x_avg_files.sort()
    y_avg_files.sort()
    e_avg_files.sort()

    # FWHM info
    fwhm_files = glob.glob('fwhm_info*')
    
    x_avg_archive = []
    y_avg_archive = []
    e_avg_archive = []
    x_avg_obs = []
    y_avg_obs = []
    e_avg_obs = []

    # Texts...
    t_avg_files = glob.glob('text_average*')
    t_avg_files.sort()
    t_img_files = glob.glob('text_image*')
    t_img_files.sort()
    
    # Titles:
    titles = glob.glob('title*')

    # Entering plotting loop:
    with PdfPages(filename) as pdf:
        for i in range(1, n_figures + 1):
            avg_flag = False
            img_flag = False
            str_flag = False
            idx = str(i).zfill(2)
            # Average curve:
            try:
                x_avg_file = [f for f in x_avg_files if idx in f][0]
                x_avg = np.loadtxt(x_avg_file)
                x_avg_archive.append(x_avg)
                x_avg_obs.append(x_avg)
                str_flag = True
            except (FileNotFoundError, IndexError) as e:
                #if os.path.isfile('xdatacal_{}'.format(idx)) and os.path.isfile('xdatacal_{}'.format(str(i + 1).zfill(2))):
                if os.path.isfile('xdatamean_{}'.format(idx)):
                    x_avg = calc_avg('xdatamean_{}'.format(idx))
                    x_avg_obs = []
                    avg_flag = True
                #elif os.path.isfile('xdatacal_{}'.format(idx)) and not os.path.isfile('xdatacal_{}'.format(str(i + 1).zfill(2))):
                elif os.path.isfile('xdatafinal_{}'.format(str(i + 1).zfill(2))):
                    x_avg = calc_avg('xdatafinal_{}'.format(str(i + 1).zfill(2)))
                    img_flag = True
                else:
                    print('Could not open xdatadots{}.'.format(idx))
                    return None
            try:
                y_avg_file = [f for f in y_avg_files if idx in f][0]
                y_avg = np.loadtxt(y_avg_file)
                y_avg_archive.append(y_avg)
                y_avg_obs.append(y_avg)
            except (IndexError, FileNotFoundError) as e:
                #if os.path.isfile('ydatacal_{}'.format(idx)) and os.path.isfile('ydatacal_{}'.format(str(i + 1).zfill(2))):
                if os.path.isfile('ydatamean_{}'.format(idx)):
                    y_avg = calc_avg('ydatamean_{}'.format(idx))
                    y_avg_obs = []
                #elif os.path.isfile('ydatacal_{}'.format(idx)) and not os.path.isfile('ydatacal_{}'.format(str(i + 1).zfill(2))):
                elif os.path.isfile('ydatafinal_{}'.format(str(i + 1).zfill(2))):
                    y_avg = calc_avg('ydatafinal_{}'.format(str(i + 1).zfill(2)))
                else:
                    print('Could not open ydatadots{}.'.format(idx))
                    return None
            try:
                e_avg_file = [f for f in e_avg_files if idx in f][0]
                err = np.loadtxt(e_avg_file)
                e_avg_obs.append(err)
                e_avg_archive.append(err)
            except (FileNotFoundError, IndexError) as e:
                #if os.path.isfile('xdatacal_{}'.format(idx)) and os.path.isfile('edatacal_{}'.format(str(i + 1).zfill(2))):
                if os.path.isfile('edatamean_{}'.format(idx)):
                    err = calc_avg('edatamean_{}'.format(idx))
                    e_avg_obs = []
                #elif os.path.isfile('xdatacal_{}'.format(idx)) and not os.path.isfile('edatacal_{}'.format(str(i + 1).zfill(2))):
                elif os.path.isfile('edatafinal_{}'.format(str(i + 1).zfill(2))):
                    err = calc_avg('edatafinal_{}'.format(str(i + 1).zfill(2)))
                else:
                    print('Could not open edatadots. All errros will be zero.')
                    err = np.zeros(len(y_avg))


            # Accumulative sum:
            try:
                x_sum_file = [f for f in x_sum_files if idx in f][0]
                x_sum = np.loadtxt(x_sum_file)
            except FileNotFoundError:
                print('Could not open xdatasum_{}.'.format(i))
                return None
            try:
                y_sum_file = [f for f in y_sum_files if idx in f][0]
                y_sum = np.loadtxt(y_sum_file)
            except FileNotFoundError:
                print('Could not open ydatasum_{}.'.format(i))
                return None


            # Calibration curve:
            try:
                x_cal_file = [f for f in x_cal_files if idx in f][0]
                x_cal = np.loadtxt(x_cal_file)
            except FileNotFoundError:
                print('Could not open xdatacal{}.'.format(i))
                return None
            try:
                y_cal_file = [f for f in y_cal_files if idx in f][0]
                y_cal = np.loadtxt(y_cal_file)
            except FileNotFoundError:
                print('Could not open ydatacal{}.'.format(i))
                return None

            
            # cleaning negative data
            extra_args = np.where(x_avg < 0)
            x_avg = np.delete(x_avg, extra_args)
            y_avg = np.delete(y_avg, extra_args)
            err = np.delete(err, extra_args)

            n_sources = int(re.findall(r'\d+', x_avg_file)[1])
    
    
            # plotting data
            plt.Figure(figsize = (14, 9))
            plt.plot(x_sum, y_sum, label = 'Fraction of total counts')
            plt.plot(x_cal, [1] * len(x_cal), '--', color = 'black')
            plt.plot(x_cal, y_cal, '--', label = 'Calibration curve')
            plt.plot(x_avg, y_avg, color = 'black')
            plt.xlim(right = max(x_sum))
            
            # Make adjustments depending on the type of plot:
            if avg_flag:
                plt.errorbar(x_avg, y_avg, yerr = err, fmt = '.', label = 'Average of {0} sources.'.format(n_sources))
                try:
                    with open('fwhm_info_apf_{}'.format(str(i + 1).zfill(2))) as fw:
                        fwhm_flag = True
                        fw_info = fw.readlines()[0]
                except FileNotFoundError:
                    fwhm_flag = False
                    warnings.warn('Could not open fwhm_info_apf_{}.'.format(str(i + 1).zfill(2)))

           ##if img_flag:
#                plt.errorbar(x_avg, y_avg, yerr = err, fmt = '.', label = 'Average of {0} sources.'.format(n_sources)) 
                #try:
                #    with open('fwhm_info_apf_{}'.format(str(i + 1).zfill(2))) as fw:
                #        fwhm_flag = True
                #        fw_info = fw.readlines()[0]
                #except FileNotFoundError:
                    #fwhm_flag = False
                    #warnings.warn('Could not open fwhm_info_apf_{}.'.format(str(i + 1).zfill(2)))

            if str_flag:
                (_, caps, _) = plt.errorbar(x_avg, y_avg, yerr = err, fmt = '.', capsize = 10, markersize = 8, label = 'Average of {0} sources.'.format(n_sources))
                for cap in caps:
                    cap.set_markeredgewidth(0.1)
                try:
                    with open('fwhm_info_str_{}'.format(idx)) as fw:
                        fwhm_flag = True
                        fw_info = fw.readlines()[0]
                except FileNotFoundError:
                    fwhm_flag = False
                    warnings.warn('Could not open fwhm_info_str_{}.'.format(idx))

            
            if fwhm_flag:
                plt.text(0.1, 1.01, fw_info, fontsize = 7)
            
            # Labels and info
            plt.legend()
            plt.xlabel('Radius (pixels)')
            for j in titles:
                try:
                    if idx in j:
                        with open(j) as title_file:
                            title_fig = title_file.readlines()[0].rstrip().strip()
                except FileNotFoundError:
                    warnings.warn('Could not load a title for figure {}.'.format(idx))
                    title_fig = '--'
            plt.suptitle(title_fig, fontsize = 9)
            #if i == num_dirs and num_dirs > 1:
            #    plt.title('... Filter {0} from {1} stars.'.format(om_filter, num_dirs))
            #else:
            #    plt.title(plot_title)
            # RDS # filename = filename.replace('ps', 'pdf')
            plt.ylabel('Azimuthally averaged normalised counts.')
            plt.ylim(bottom = 0, top = 1)
            pdf.savefig()
            plt.close() 
        
        if os.path.isfile('datahist'):
            plt.Figure(figsize = (14, 9))
            psf_widths = np.loadtxt('datahist')
            plt.hist(psf_widths, weights = np.ones(len(psf_widths)) / len(psf_widths))
            with open('title_hist') as th:
                title_hist = th.readlines()[0].rstrip().strip()
            plt.title(title_hist)
            with open('final_hist') as fh:
                text_hist = fh.readlines()[0].rstrip().strip()
            plt.text(0, 1, text_hist, fontsize = 10)
            plt.ylabel('Frequency (%) (N = {}'.format(n_sources))
            plt.xlabel('FWHM (arcsec)')
            plt.gca().yaxis.set_major_formatter(PercentFormatter(1))
            pdf.savefig()
            os.remove('datahist')
            plt.close()
        

        # Complete informatino for the metadata.
        d = pdf.infodict()
        d['Title'] = filename
        d['Author'] = 'ompsfprofile, XMM-Newton'
        d['Subject'] = 'Optical Monitor profile analysis.'
        d['Keywords'] = 'ESA XMM-Newton OM ompsfprofile'
        d['CreationDate'] = datetime.datetime.now()
        d['ModDate'] = datetime.datetime.now()
    
    # Cleaning Up...
    for f in x_avg_files:
        os.remove(f)
    for f in y_avg_files:
        os.remove(f)
    for f in y_cal_files:
        os.remove(f)
    for f in x_cal_files:
        os.remove(f)
    for f in x_sum_files:
        os.remove(f)
    for f in y_sum_files:
        os.remove(f)
    for f in fwhm_files:
        os.remove(f)
    for f in e_avg_files:
        os.remove(f)
    for f in mean_files:
        os.remove(f)
    for f in final_files:
        os.remove(f)
    if os.path.isfile('text_average'):
        os.remove('text_average')
    for f in titles:
        os.remove(f)
    if os.path.isfile('final_hist'):
        os.remove('final_hist')


    return None


if __name__ == '__main__':
    filename = sys.argv[1].replace('.ps', '.pdf')

    main(filename)
